import asyncio

# import objgraph
# import copy
import functools
import random
import sys
import time
from typing import Any, Generator, Tuple, Union

from graia.broadcast import Broadcast
from graia.broadcast.builtin.decorators import Depend
from graia.broadcast.entities.decorator import Decorator
from graia.broadcast.entities.dispatcher import BaseDispatcher
from graia.broadcast.entities.event import Dispatchable
from graia.broadcast.entities.exectarget import ExecTarget
from graia.broadcast.entities.listener import Listener
from graia.broadcast.exceptions import ExecutionStop, PropagationCancelled
from graia.broadcast.interfaces.decorator import DecoratorInterface
from graia.broadcast.interfaces.dispatcher import DispatcherInterface
from graia.broadcast.interrupt import InterruptControl
from graia.broadcast.interrupt.waiter import Waiter
from graia.broadcast.utilles import dispatcher_mixin_handler


class TestDispatcher(BaseDispatcher):
    @staticmethod
    async def catch(interface: DispatcherInterface):
        if interface.name == "ster":
            return 1


class TestEvent1(Dispatchable):
    class Dispatcher(BaseDispatcher):
        @staticmethod
        async def catch(interface: "DispatcherInterface"):
            if interface.name == "ster":
                return "1"
            elif interface.name == "ster1":
                return 54352345


class TestEvent2(Dispatchable):
    class Dispatcher(BaseDispatcher):
        @staticmethod
        async def catch(interface: "DispatcherInterface"):
            if interface.name == "ster":
                return 6546


event = TestEvent1()
loop = asyncio.new_event_loop()

broadcast = Broadcast()


@broadcast.receiver(TestEvent1)
async def s(e: TestEvent1):
    broadcast.postEvent(TestEvent2(), e)


@broadcast.receiver(TestEvent2)
async def t(e: TestEvent2, ster, ster1):
    print(e, ster, ster1)


loop.run_until_complete(asyncio.wait([broadcast.postEvent(event)]))



import asyncio
from typing import Annotated

from graia.broadcast import Broadcast, Dispatchable
from graia.broadcast.entities.dispatcher import BaseDispatcher
from graia.broadcast.interfaces.dispatcher import DispatcherInterface


class ExampleEvent(Dispatchable):
    class Dispatcher(BaseDispatcher):
        @staticmethod
        async def catch(interface: "DispatcherInterface"):
            if interface.annotation is str:
                return "ok, i'm."


loop = asyncio.get_event_loop()
broadcast = Broadcast()


async def test_derive_1(v: str, dii: DispatcherInterface):
    print("in derive 1", v)
    return v[1:]


@broadcast.receiver("ExampleEvent")  # or just receiver(ExampleEvent)
async def event_listener(maybe_you_are_str: Annotated[str, test_derive_1, test_derive_1]):
    print(maybe_you_are_str)  # <<< ok, i'm


async def main():
    await broadcast.postEvent(ExampleEvent())  # sync call is allowed.


loop.run_until_complete(main())



import asyncio

from graia.broadcast import Broadcast
from graia.broadcast.builtin.decorators import Depend
from graia.broadcast.entities.dispatcher import BaseDispatcher
from graia.broadcast.entities.event import Dispatchable
from graia.broadcast.interfaces.dispatcher import DispatcherInterface


class TestDispatcher(BaseDispatcher):
    @classmethod
    async def beforeExecution(cls, interface: DispatcherInterface):
        if interface.depth == 0:
            print("beforeExecution")

    @classmethod
    async def catch(cls, interface: DispatcherInterface):
        if interface.name == "ster":
            return 1


class TestEvent(Dispatchable):
    class Dispatcher(BaseDispatcher):
        mixin = [TestDispatcher]

        @classmethod
        async def catch(cls, interface: "DispatcherInterface"):
            if interface.name == "ster1":
                return "1"
            elif interface.name == "ster2":
                return 54352345

        @classmethod
        async def afterExecution(
            self,
            interface: DispatcherInterface,
            *args,
        ):
            if interface.depth == 0:
                print("afterExecution")


event = TestEvent()
loop = asyncio.new_event_loop()

broadcast = Broadcast()


@broadcast.receiver(
    TestEvent,
    decorators=[
        Depend(lambda ster: print("depend: ster", ster)),
        Depend(lambda ster1: print("depend: ster1", ster1)),
        Depend(lambda ster2: print("depend: ster2", ster2)),
    ],
)
async def s(e: TestEvent):
    print(e)


def error(a: int):
    raise ValueError("error", a)


@broadcast.receiver(TestEvent, decorators=[Depend(error)])
async def s1(e: TestEvent):
    print(e)


loop.run_until_complete(asyncio.wait([broadcast.postEvent(event)]))
